import Fanfou from './fanfou.js';
import * as api from './api.js';
import User from './user.js';
import Photo from './photo.js';
export declare type StatusBoldText = {
    text: string;
    isBold: boolean;
};
export declare type StatusEntityText = {
    type: 'text';
    text: string;
    boldTexts?: StatusBoldText[];
};
export declare type StatusEntityTag = {
    type: 'tag';
    text: string;
    query: string;
    boldTexts?: StatusBoldText[];
};
export declare type StatusEntityAt = {
    type: 'at';
    text: string;
    name: string;
    id: string;
    boldTexts?: StatusBoldText[];
};
export declare type StatusEntityLink = {
    type: 'link';
    text: string;
    link: string;
    boldTexts?: StatusBoldText[];
};
export declare type StatusEntity = StatusEntityText | StatusEntityTag | StatusEntityAt | StatusEntityLink;
export declare type StatusReplyOptions = api.CreateStatusOptions;
export declare type StatusRepostOptions = api.CreateStatusOptions;
export declare type StatusFavoriteOptions = {
    id?: string;
    mode?: api.APIMode;
    format?: api.APIFormat;
    callback?: string;
};
export declare type StatusUnfavoriteOptions = {
    id?: string;
    mode?: api.APIMode;
    format?: api.APIFormat;
};
export declare type StatusDestroyOptions = {
    id?: string;
    mode?: api.APIMode;
    format?: api.APIFormat;
    callback?: string;
};
export declare type StatusContextOptions = {
    id?: string;
    mode?: api.APIMode;
    format?: api.APIFormat;
    callback?: string;
};
declare class Status {
    createdAt: string;
    id: string;
    rawid: number;
    text: string;
    source: string;
    truncated: boolean;
    inReplyToStatusId?: string;
    inReplyToUserId?: string;
    inReplyToScreenName?: string;
    inReplyToLastmsgId?: string;
    favorited: boolean;
    isSelf?: boolean;
    location?: string;
    repostStatus?: Status;
    repostStatusId?: string;
    repostUserId?: string;
    repostScreenName?: string;
    type?: string;
    sourceUrl?: string;
    sourceName?: string;
    plainText?: string;
    user?: User;
    photo?: Photo;
    entities?: StatusEntity[];
    private readonly ff;
    constructor(ff: Fanfou, status: any);
    isReply(): boolean;
    isRepost(): boolean | "" | undefined;
    isOrigin(): boolean;
    isOriginRepost(): false | RegExpMatchArray | null;
    reply: (options: StatusReplyOptions) => Promise<Status>;
    repost: (options?: api.CreateStatusOptions | undefined) => Promise<Status>;
    favorite: (options?: StatusFavoriteOptions | undefined) => Promise<Status>;
    unfavorite: (options?: StatusUnfavoriteOptions | undefined) => Promise<Status>;
    destroy: (options?: StatusDestroyOptions | undefined) => Promise<User>;
    context: (options?: StatusContextOptions | undefined) => Promise<Status[]>;
    private getType;
}
export default Status;
