import he from 'he';
import Status from './status.js';
import User from './user.js';
import Trend from './trend.js';
import DirectMessage from './direct-message.js';
export const uriType = (uri) => {
    const uriList = {
        // Timeline
        '/search/public_timeline': 'timeline',
        '/search/user_timeline': 'timeline',
        '/photos/user_timeline': 'timeline',
        '/statuses/friends_timeline': 'timeline',
        '/statuses/home_timeline': 'timeline',
        '/statuses/public_timeline': 'timeline',
        '/statuses/replies': 'timeline',
        '/statuses/user_timeline': 'timeline',
        '/statuses/context_timeline': 'timeline',
        '/statuses/mentions': 'timeline',
        '/favorites': 'timeline',
        // Status
        '/statuses/update': 'status',
        '/statuses/show': 'status',
        '/favorites/destroy': 'status',
        '/favorites/create': 'status',
        '/photos/upload': 'status',
        // Users
        '/users/tagged': 'users',
        '/users/followers': 'users',
        '/users/friends': 'users',
        '/friendships/requests': 'users',
        // User
        '/users/show': 'user',
        '/friendships/create': 'user',
        '/friendships/destroy': 'user',
        '/account/verify_credentials': 'user',
        // Conversation
        '/direct_messages/conversation': 'conversation',
        '/direct_messages/inbox': 'conversation',
        '/direct_messages/sent': 'conversation',
        // Conversation List
        '/direct_messages/conversation_list': 'conversation-list',
        // Direct Message
        '/direct_messages/new': 'dm',
        '/direct_messages/destroy': 'dm',
        // Saved Search
        '/saved_searches/create': 'saved-search',
        '/saved_searches/destroy': 'saved-search',
        '/saved_searches/show': 'saved-search',
        // Saved Search List
        '/saved_searches/list': 'saved-search-list',
        // Trend List
        '/trends/list': 'trend-list'
    };
    // @ts-expect-error
    const type = uriList[uri] || null;
    if (!type && /^\/favorites\/(?:create|destroy)\/.+/.test(uri)) {
        return 'status';
    }
    return type;
};
export const parseList = (ff, list, type) => {
    switch (type) {
        case 'timeline':
            return list.map((item) => item instanceof Status ? item : new Status(ff, item));
        case 'users':
            return list.map((item) => item instanceof User ? item : new User(ff, item));
        case 'conversation':
            return list.map((item) => item instanceof DirectMessage ? item : new DirectMessage(ff, item));
        case 'conversation-list':
            return list.map((item) => {
                item.dm =
                    item.dm instanceof DirectMessage
                        ? item.dm
                        : new DirectMessage(ff, item.dm);
                return item;
            });
        case 'saved-search-list':
            return list.map((item) => item instanceof Trend ? item : new Trend(ff, item));
        case 'trend-list': {
            list.trends = list.trends.map((item) => item instanceof Trend ? item : new Trend(ff, item));
            return list;
        }
        default:
            return list;
    }
};
export const parseData = (ff, data, type) => {
    switch (type) {
        case 'timeline':
        case 'users':
        case 'conversation':
        case 'conversation-list':
        case 'saved-search-list':
        case 'trend-list':
            return parseList(ff, data, type);
        case 'status':
            return data instanceof Status ? data : new Status(ff, data);
        case 'user':
            return data instanceof User ? data : new User(ff, data);
        case 'dm':
            return data instanceof DirectMessage ? data : new DirectMessage(ff, data);
        case 'saved-search':
            return data instanceof Trend ? data : new Trend(ff, data);
        default:
            return data;
    }
};
export const hasBold = (text) => {
    return /<b>[\s\S\n]*?<\/b>/g.test(text);
};
export const removeBoldTag = (text) => {
    return text.replace(/<b>/g, '').replace(/<\/b>/g, '');
};
export const getBoldTexts = (text) => {
    var _a, _b, _c;
    const pattern = /<b>[\s\S\n]*?<\/b>/g;
    let theText = text;
    const match = text.match(pattern);
    const textArray = [];
    if (match) {
        for (const item of match) {
            const index = theText.indexOf(item);
            if (index > 0) {
                const t = theText.slice(0, index);
                textArray.push({
                    text: he.decode(t),
                    isBold: false
                });
            }
            const match = (_a = /<b>(?<t>[\s\S\n]*?)<\/b>/.exec(item)) !== null && _a !== void 0 ? _a : {
                groups: { t: '' }
            };
            textArray.push({
                // eslint-disable-next-line @typescript-eslint/dot-notation
                text: he.decode((_c = (_b = match.groups) === null || _b === void 0 ? void 0 : _b['t']) !== null && _c !== void 0 ? _c : ''),
                isBold: true
            });
            theText = theText.slice(index + item.length);
        }
        if (theText.length > 0) {
            textArray.push({
                text: he.decode(theText),
                isBold: false
            });
        }
        return textArray;
    }
    return [
        {
            text: he.decode(text),
            isBold: false
        }
    ];
};
export const getEntities = (statusText) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q;
    const pattern = /[@#]?<a href=".*?".*?>[\s\S\n]*?<\/a>#?/g;
    const tagPattern = /#<a href="\/q\/(?<link>.*?)".?>(?<tag>[\s\S\n]*)<\/a>#/;
    const atPattern = /@<a href="(?:http|https):\/\/[.a-z\d-]*fanfou.com\/(?<id>.*?)".*?>(?<at>.*?)<\/a>/;
    const linkPattern = /<a href="(?<link>.*?)".*?>(?<text>.*?)<\/a>/;
    const match = statusText.match(pattern);
    const entities = [];
    let theText = statusText;
    if (match) {
        for (const item of match) {
            const index = theText.indexOf(item);
            // Text
            if (index > 0) {
                const text = theText.slice(0, index);
                const originText = he.decode(removeBoldTag(theText.slice(0, index)));
                const thisEntity = {
                    type: 'text',
                    text: originText
                };
                if (hasBold(text)) {
                    thisEntity.boldTexts = getBoldTexts(text);
                }
                entities.push(thisEntity);
            }
            // Tag
            if (item.startsWith('#') && tagPattern.test(item)) {
                const matchText = (_a = tagPattern.exec(item)) !== null && _a !== void 0 ? _a : {
                    groups: { tag: '', link: '' }
                };
                const text = `#${(_b = matchText.groups) === null || _b === void 0 ? void 0 : _b.tag}#`;
                const originText = he.decode(removeBoldTag(text));
                const thisEntity = {
                    type: 'tag',
                    text: originText,
                    query: decodeURIComponent(he.decode((_d = (_c = matchText.groups) === null || _c === void 0 ? void 0 : _c.link) !== null && _d !== void 0 ? _d : ''))
                };
                if (hasBold(text)) {
                    thisEntity.boldTexts = getBoldTexts(text);
                }
                entities.push(thisEntity);
            }
            // At
            if (item.startsWith('@') && atPattern.test(item)) {
                const matchText = (_e = atPattern.exec(item)) !== null && _e !== void 0 ? _e : { groups: { at: '', id: '' } };
                const text = `@${(_f = matchText.groups) === null || _f === void 0 ? void 0 : _f.at}`;
                const originText = he.decode(removeBoldTag(text));
                const thisEntity = {
                    type: 'at',
                    text: originText,
                    name: he.decode((_h = (_g = matchText.groups) === null || _g === void 0 ? void 0 : _g.at) !== null && _h !== void 0 ? _h : ''),
                    id: (_k = (_j = matchText.groups) === null || _j === void 0 ? void 0 : _j.id) !== null && _k !== void 0 ? _k : ''
                };
                if (hasBold(text)) {
                    thisEntity.boldTexts = getBoldTexts(text);
                }
                entities.push(thisEntity);
            }
            // Link
            if (item.startsWith('<') && linkPattern.test(item)) {
                const matchText = (_l = linkPattern.exec(item)) !== null && _l !== void 0 ? _l : {
                    groups: { link: '', text: '' }
                };
                const link = (_o = (_m = matchText.groups) === null || _m === void 0 ? void 0 : _m.link) !== null && _o !== void 0 ? _o : '';
                const text = (_q = (_p = matchText.groups) === null || _p === void 0 ? void 0 : _p.text) !== null && _q !== void 0 ? _q : '';
                const originText = removeBoldTag(text);
                const thisEntity = {
                    type: 'link',
                    text: originText,
                    link
                };
                if (hasBold(text)) {
                    thisEntity.boldTexts = getBoldTexts(text);
                }
                entities.push(thisEntity);
            }
            theText = theText.slice(index + item.length);
        }
        if (theText.length > 0) {
            const text = theText;
            const originText = he.decode(removeBoldTag(text));
            const thisEntity = {
                type: 'text',
                text: originText
            };
            if (hasBold(text)) {
                thisEntity.boldTexts = getBoldTexts(text);
            }
            entities.push(thisEntity);
        }
        return entities;
    }
    const text = theText;
    const originText = he.decode(removeBoldTag(theText));
    const thisEntity = {
        type: 'text',
        text: originText
    };
    if (hasBold(text)) {
        thisEntity.boldTexts = getBoldTexts(text);
    }
    return [thisEntity];
};
export const getSourceUrl = (source) => {
    var _a, _b;
    const matched = /<a href="(?<link>.*)" target="_blank">.+<\/a>/.exec(source);
    // eslint-disable-next-line @typescript-eslint/dot-notation
    return (_b = (_a = matched === null || matched === void 0 ? void 0 : matched.groups) === null || _a === void 0 ? void 0 : _a['link']) !== null && _b !== void 0 ? _b : '';
};
export const getSourceName = (source) => {
    var _a, _b;
    const matched = /<a href=".*" target="_blank">(?<name>.+)<\/a>/.exec(source);
    // eslint-disable-next-line @typescript-eslint/dot-notation
    return (_b = (_a = matched === null || matched === void 0 ? void 0 : matched.groups) === null || _a === void 0 ? void 0 : _a['name']) !== null && _b !== void 0 ? _b : source;
};
export const getPlainText = (entities) => {
    let text = '';
    for (const t of entities !== null && entities !== void 0 ? entities : []) {
        text += t.text;
    }
    return he.decode(text);
};
